//
// 30F6014A ADC12 routines
//
// OH2NLT / Juha Niinikoski 28.03.2009
//

// MPX4115A MAP scaling 29.03.2009

// scaling factors
// battery voltage
#define BATT_SCALE	3539L				// scaling factor * 65536

// MAP MPX4115A sensor, range 15 to 115 kPa
// ZERO = -389 ADC counts
// 100kPa = 3297 ADC counts
// SPAN = 3297 - (-389) = 3686
#define MAP_OFFSET 389					// add to the ADC value
#define MAP_SCALE_HPA	17780			// 1000/SPAN * 65536
#define MAP_SCALE_KPA	1778			// 100/SPAN * 65536

//Includes
#include "cop_ignition.h"						// board & processor specific definitions

// Init ADC system
void init_adc12(void)
	{
	ADCSSL = 0x0000;					// scan select
	ADPCFG = 0x1FFF;					// AN13 to AN15, in use, bit 0 = select
	ADCHS = 0x000F;						// select CH15, Vref- for MUXA
//	ADCON3 = 0x0080;					// internal RC clock
	ADCON3 = 0x0F80;					// internal RC clock, sample time = 15TAD
//	ADCON2 = 0x2000;					// use MUXA, do not scan inputs, Ext Vref+
	ADCON2 = 0x0000;					// use MUXA, do not scan inputs, VCC = Vref+ = 5V
//	ADCON1 = 0x8000;					// ADON, format = integer, manual sampling, manual start
	ADCON1 = 0x00E0;					// ADON, format = integer, auto sampling / start
	ADCON1bits.ADON = 1;				// start A/D system		
	}


// convert selected channell
int convert_adc12(unsigned int adchs)
	{
	ADCON1bits.DONE = 0;				// clear done flag
	ADCHS = (adchs & 0x000F);			// select CH, Vref- for MUXA
	ADCON1bits.SAMP = 1;				// start sampling
//	us_delay(500);
//	ADCON1bits.SAMP = 0;				// start conversion
	while(ADCON1bits.DONE == 0);		// wait for ready
	return(ADCBUF0);					// return with buff#0
	}


// scaled voltage, 120 = 12.0
int scale_batt_v(int adc_val)
	{
	long ltemp;
	ltemp = BATT_SCALE * (long)adc_val;
	return(ltemp >> 16);
	}

// scaled pressure hPa
int scale_map_hpa(int adc_val)
	{
	long ltemp;
	ltemp = MAP_SCALE_HPA * (long)(adc_val + MAP_OFFSET);
	return(ltemp >> 16);
	}

// scaled pressure kPa
int scale_map_kpa(int adc_val)
	{
	long ltemp;
	ltemp = MAP_SCALE_KPA * (long)(adc_val + MAP_OFFSET);
	return(ltemp >> 16);
	}
