#ifndef __COP_H
#define __COP_H

//
// Suzuki COP / DIS ignition
// Juha Niinikoski, OH2NLT 26.03.2009
//
// 
// Microchip MPLAB + C30 compiler
// MCU dsPIC30F6014A
//

// Board specific definitions

// processor specific definitions
#include <p30f6014A.h>
//

//#define RISING_CAM_SYNC					// rising edge cranc pulse CAM sync
#define FALLING_CAM_SYNC				// rising edge cranc pulse CAM sync

#define BASE_ADVANCE 60					// base advance (0.1 degrees)

// system cycle rate, XTAL*PLL/4, PLL = 4, 29,5 MHz clock
//#define FCY	7372800UL					// 7.3728MHz

// system cycle rate, XTAL*PLL/4, PLL = 8, 58,9 MHz clock
#define FCY	14745600UL

// system cycle rate, XTAL*PLL/4, PLL = 16, 117.968MHz clock
//#define FCY	29491200UL


#define DLYCONST ((FCY/1000000UL) + 1)	// constant for timer delay routines


// I/O port init values
#define		TRISA_INIT	0xF000			// INT pins 0 INPUTS
#define		PORTA_INIT	0x0000

#define		TRISB_INIT	0xE000			// ADC pins = inputs
#define		PORTB_INIT	0x0000

#define		TRISC_INIT	0x4000			// T1CK = in
#define		PORTC_INIT	0x0000

#define		TRISD_INIT	0x0000
#define		PORTD_INIT	0x0000

#define		TRISF_INIT	0x0040			// Flasher_start = input
#define		PORTF_INIT	0x0000

#define		TRISG_INIT	0x0000
#define		PORTG_INIT	0x0000	


// I/O registers & bits

// A/D converter
#define		MAP_CH		15			// MAP pressure CH#
#define		BATT_CH		14			// Battery voltage CH#
#define		AUX_CH		13			// Spare

// inputs
#define		CRANC		_RA15		// CRANC sensor / INT 3+4 / T1CK
#define		CAM			_RA12		// CAM sensor / INT 1
#define		SPARE_IN	_RA13		// SPARE input / INT 2

#define		SW1			_RF6		// Flasher boot switch

// outputs
#define		ECM_OUT		_LATD6		// ECM ignition pulse out
#define		TACHO_OUT	_LATD7		// Tachometer ignition pulse out

#define		TP1			_LATG15		// TP / Pin 1
#define		TP80		_LATG13		// TP / Pin 80

// Status LEDs
#define LD1				_LATB6		// RED led
#define LD2				_LATB7		// GREEN led

// coil outputs
#define COIL_PORT		PORTD		// Coil outouts
#define COIL_MASK		0x000F		// Coil bits
#define	COIL_1			_LATD0		// Coil #1 output
#define	COIL_2			_LATD1		// Coil #2 output
#define	COIL_3			_LATD2		// Coil #3 output
#define	COIL_4			_LATD3		// Coil #4 output

#endif
