//
// Suzuki COP / DIS ignition
// Juha Niinikoski, OH2NLT 28.07.2009
//
// 
// Microchip MPLAB + C30 compiler
// MCU dsPIC30F6014A
//
// Data logger module


#define MAP_ROWS	11		// number of MAP entries
#define RPM_COLS	17		// number of RPM columns

const int map_steps[MAP_ROWS] = {0, 20, 30, 40, 50, 60, 70, 80, 90, 100, 110};
const int rpm_steps[RPM_COLS] = {0, 500, 750, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500};

int last_sample[MAP_ROWS][RPM_COLS];			// last single sample table
unsigned long sample_accu[MAP_ROWS][RPM_COLS];	// sample accumulator
unsigned int sample_amount[MAP_ROWS][RPM_COLS];	// number of samples

int map_idx;								// table indexes
int rpm_idx;


extern unsigned char pbuf[100];				// sprintf buffer


// log data
void log_data(int rpm, int kpa, int adv)
	{
	int x;
	x = MAP_ROWS - 1;						// find & set map index
	while(map_steps[x] > kpa)
		{
		x--;
		}
	map_idx = x;

	x = RPM_COLS - 1;						// find & set rpm index
	while(rpm_steps[x] > rpm)
		{
		x--;
		}
	rpm_idx = x;

	last_sample[map_idx][rpm_idx] = adv;								// log data
	sample_accu[map_idx][rpm_idx] = sample_accu[map_idx][rpm_idx] + adv;	// accumulate
	sample_amount[map_idx][rpm_idx]++;									// count samples

//	sprintf(pbuf, "\n\rRPM_idx %d, MAP_idx %d, Adv %d    ",rpm_idx, map_idx, adv);
//	putst2(pbuf);
	}

// print logged last sample data
void print_logged_single_sample_data(void)
	{
	int m, r;

	sprintf(pbuf, "\n\rLogged Data, last single samples\n\rMap ");
	putst2(pbuf);

	for(m=0; m<MAP_ROWS; m++)
		{
		sprintf(pbuf, "\n\r%.3d ", map_steps[m]);
		putst2(pbuf);

		for(r=0; r<RPM_COLS; r++)
			{
			sprintf(pbuf, ",%.3d", last_sample[m][r]);
			putst2(pbuf);
			}
		}

	sprintf(pbuf, "\n\rRpm ");
	putst2(pbuf);
	for(r=0; r<RPM_COLS; r++)
		{
		sprintf(pbuf, ",%.3d", (rpm_steps[r]/10));
		putst2(pbuf);
		}
	}

// print logged average data
void print_logged_average_data(void)
	{
	int m, r, average;

	sprintf(pbuf, "\n\rLogged Average Data\n\rMap ");
	putst2(pbuf);

	for(m=0; m<MAP_ROWS; m++)
		{
		sprintf(pbuf, "\n\r%.3d ", map_steps[m]);
		putst2(pbuf);

		for(r=0; r<RPM_COLS; r++)
			{
			if(sample_amount[m][r] > 0)
				{
				average = sample_accu[m][r] / sample_amount[m][r];
				sprintf(pbuf, ",%.3d", average);
				putst2(pbuf);
				}
			else
				{
				sprintf(pbuf, ",---");
				putst2(pbuf);
				}
			}
		}

	sprintf(pbuf, "\n\rRpm ");
	putst2(pbuf);
	for(r=0; r<RPM_COLS; r++)
		{
		sprintf(pbuf, ",%.3d", (rpm_steps[r]/10));
		putst2(pbuf);
		}
	}


// print logged sample numbers
void print_logged_single_samples(void)
	{
	int m, r;

	sprintf(pbuf, "\n\rLogged samples\n\rMap ");
	putst2(pbuf);

	for(m=0; m<MAP_ROWS; m++)
		{
		sprintf(pbuf, "\n\r%.3d ", map_steps[m]);
		putst2(pbuf);

		for(r=0; r<RPM_COLS; r++)
			{
			sprintf(pbuf, ",%.3d", sample_amount[m][r]);
			putst2(pbuf);
			}
		}

	sprintf(pbuf, "\n\rRpm ");
	putst2(pbuf);
	for(r=0; r<RPM_COLS; r++)
		{
		sprintf(pbuf, ",%.3d", (rpm_steps[r]/10));
		putst2(pbuf);
		}
	}


// Clearlogged data
void clear_logged_data(void)
	{
	int m, r;

	sprintf(pbuf, "\n\rLogged Data Cleared");
	putst2(pbuf);

	for(m=0; m<MAP_ROWS; m++)
		{
		for(r=0; r<RPM_COLS; r++)
			{
			last_sample[m][r] = 0;				// clear all logger tables
			sample_accu[m][r] = 0;
			sample_amount[m][r] = 0;
			}
		}
	}
