//
// Suzuki COP / DIS ignition
// Juha Niinikoski, OH2NLT 27.08.2009
//
// 
// Microchip MPLAB + C30 compiler
// MCU dsPIC30F6014A
//

// Test Ignition Map
// stored units are crancshaft events = 1,8deg
// base advance is about 6deg

extern unsigned char pbuf[100];					// sprintf buffer

#define MAP_ROWS	11		// number of MAP entries
#define RPM_COLS	17		// number of RPM columns

const int map_steps[MAP_ROWS] = {0, 20, 30, 40, 50, 60, 70, 80, 90, 100, 110};
// 1 RPM resolution
// const int rpm_steps[RPM_COLS] = {0, 500, 750, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500};
// 10 RPM resolution
const int rpm_steps[RPM_COLS] = {0, 50, 75, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750};

// very first test table 27.08.2009
// table format [MAP_ROWS][RPM_COLS];
const int ign_map[MAP_ROWS][RPM_COLS] = {
{0, 0, 2, 6, 11, 13, 13, 14, 14, 15, 17, 17, 18, 18, 18, 19, 20}, 		// MAP 0-20kPa
{0, 0, 2, 6, 11, 13, 13, 14, 14, 15, 17, 17, 18, 18, 18, 19, 20},		// 20-30
{0, 0, 2, 6, 11, 13, 13, 14, 14, 15, 17, 17, 18, 18, 18, 19, 20},		// 30-40
{0, 0, 2, 6, 11, 13, 13, 14, 14, 15, 17, 17, 18, 18, 18, 19, 20},		// 40-50
{0, 0, 2, 8, 11, 13, 13, 14, 14, 15, 17, 17, 18, 18, 19, 19, 20},		// 50-60
{0, 0, 2, 8, 10, 13, 14, 14, 14, 15, 17, 17, 18, 18, 19, 19, 20},		// 60-70
{0, 0, 2, 8, 12, 13, 13, 13, 13, 14, 15, 16, 17, 16, 17, 18, 19},		// 70-80
{0, 0, 1, 3, 5, 10, 10, 10, 10, 10, 10, 12, 13, 13, 14, 15, 16},		// 80-90
{0, 0, 1, 5, 5, 7, 7, 8, 8, 8, 10, 10, 11, 11, 12, 12, 13}, 			// 90-100
{0, 0, 1, 5, 5, 7, 7, 8, 8, 8, 10, 10, 11, 11, 12, 12, 13},				// 100-110
{0, 0, 1, 5, 5, 7, 7, 8, 8, 8, 10, 10, 11, 11, 12, 12, 13}				// 110 -
};


// get advance value without interpolation
int get_ign_adv(int map, int rpm)
	{
	int r, c;

// find MAP row
	r = MAP_ROWS - 1;
	while(map_steps[r] > map)
		{
		r--;
		}

// find RPM column
	c = RPM_COLS - 1;
	while(rpm_steps[c] > rpm)
		{
		c--;
		}

//	sprintf(pbuf, "\n\rRow %d, Col %d, Val %d", r, c, ign_map[r][c]);
//	putst2(pbuf);

	return ign_map[r][c];
	}


// Battery voltage Dwell compensation
// input = battery voltage(100mV units)
// output dwell us

// target dwell values(us) at voltages
// 6.4, 8.0, 9.6, 11.2, 12.8, 14.4, 16.0, 17.6

const unsigned int target_dwell_table[8]={6000, 5000, 3800, 3000, 2800, 2200, 1800, 1500};

unsigned int get_dwell(int batv)
	{
	batv = batv >> 4;				// fast divide by 16
	batv = batv - 4;				// subtract 6.4V offset
	if(batv < 0)
		batv = 0;					// keep index in limits
	if(batv > 7)
		batv = 7;

	return target_dwell_table[batv]; // return with dwell(us)
	}
 
